from abc import ABCMeta, abstractmethod
from enum import Enum, auto, unique

from pandas import DataFrame


@unique
class Allergen(Enum):
    EGG = "egg"
    WHEAT = "wheat"
    MILK = "milk"
    PEANUT = "peanut"
    BUCKWHEAT = "buckwheat"
    SHRIMP = "shrimp"
    CRAB = "crab"
    SQUID = "squid"
    SALMON_ROE = "salmonRoe"
    SALMON = "salmon"
    MACKEREL = "mackerel"
    BEEF = "beef"
    CHICKEN = "chicken"
    PORK = "pork"
    SOYBEAN = "soybean"
    ORANGE = "orange"
    APPLE = "apple"
    GELATIN = "gelatin"
    WALNUT = "walnut"
    BANANA = "banana"
    KIWI = "kiwi"
    YAM_OR_SWEET_POTATO = "yamOrSweetPotato"
    PEACH = "peach"
    SESAME = "sesame"
    ABALONE = "abalone"
    CASHEW_NUT = "cashewNut"
    MATSUTAKE_MUSHROOM = "matsutakeMushroom"

    @classmethod
    def values(cls):
        return [enum.value for enum in list(Allergen)]


class AllergenState(Enum):
    CONTAIN = auto()  # 含む
    MAY_CONTAIN = auto()  # 製造工程で混入する可能性あり
    NOT_CONTAIN = auto()  # 含まない

    def __str__(self):
        return self.name


class BaseParser(metaclass=ABCMeta):
    @abstractmethod
    def parse(self, filename: str) -> DataFrame:
        raise NotImplementedError

    @abstractmethod
    def allergen_state_of(self, value: str) -> AllergenState:
        raise NotImplementedError




import re

import numpy
import pandas as pd
import tabula

from .base import AllergenState, BaseParser, Allergen
from .util import removeNewLine

REMOTE_URL = "http://www.kura-corpo.co.jp/common/pdf/kura_allergen.pdf"
BASE_LAYOUT = [
    ("卵", Allergen.EGG.value, True),  # header, key, required
    ("小麦", Allergen.WHEAT.value, True),
    ("乳・乳製品", Allergen.MILK.value, True),
    ("落花生ピーナッツ", Allergen.PEANUT.value, True),
    ("ソバ", Allergen.BUCKWHEAT.value, True),
    ("えび", Allergen.SHRIMP.value, True),
    ("かに", Allergen.CRAB.value, True),
    ("いか", Allergen.SQUID.value, True),
    ("いくら", Allergen.SALMON_ROE.value, True),
    ("さけ", Allergen.SALMON.value, True),
    ("さば", Allergen.MACKEREL.value, True),
    ("牛肉", Allergen.BEEF.value, True),
    ("鶏肉", Allergen.CHICKEN.value, True),
    ("豚肉", Allergen.PORK.value, True),
    ("大豆", Allergen.SOYBEAN.value, True),
    ("オレンジ", Allergen.ORANGE.value, True),
    ("りんご", Allergen.APPLE.value, True),
    ("ゼラチン", Allergen.GELATIN.value, True),
    ("クルミ", Allergen.WALNUT.value, True),
    ("バナナ", Allergen.BANANA.value, True),
    ("キウイ", Allergen.KIWI.value, True),
    ("山芋", Allergen.YAM_OR_SWEET_POTATO.value, True),
    ("モモ", Allergen.PEACH.value, True),
    ("ごま", Allergen.SESAME.value, True),
    ("あわび", Allergen.ABALONE.value, True),
    ("カシューナッツ", Allergen.CASHEW_NUT.value, True),
    ("松茸", Allergen.MATSUTAKE_MUSHROOM.value, True),
]
PAGE1_LAYOUT = [
    ("品名", "name", True),
    ("(kカ一cロ皿aリ当lーり)", "calory", False),
    ("該当なし", "none", False),
    *BASE_LAYOUT,
]
PAGE2_LAYOUT = PAGE1_LAYOUT
PAGE3_LAYOUT = [
    ("品名", "name", True),
    ("一(皿kカ・cロ一aリlー杯当)り", "calory", False),
    ("該当なし", "none", False),
    *BASE_LAYOUT,
]
PAGE4_LAYOUT = [("品名", "name", True), ("該当なし", "none", False), *BASE_LAYOUT]
PAGES = [
    ("1", "定番寿司", PAGE1_LAYOUT),
    ("2", "限定商品", PAGE2_LAYOUT),
    ("3", "サイドメニュー", PAGE3_LAYOUT),
    ("4", "その他", PAGE4_LAYOUT),
]


class KuraAllergyParser(BaseParser):
    def parse(self, filename: str = REMOTE_URL) -> pd.DataFrame:
        df_list = []
        for page, category, layout in PAGES:
            df = self._parse_table(filename, page, category, layout)
            df_list.append(df)

        df_all = pd.concat(df_list, ignore_index=True)

        for _, key, _ in BASE_LAYOUT:
            df_all[key] = df_all[key].map(self.allergen_state_of)

        return df_all

    def _parse_table(
        self, filename: str, page: int, category: str, layout: list
    ) -> pd.DataFrame:

        df = tabula.read_pdf(filename, pages=str(page), lattice=True)

        columns = [re.sub(r"\s", "", column) for column in df.columns]
        expected_columns = [column[0] for column in layout]
        assert columns == expected_columns, f"{filename} のテーブルレイアウトが変更されました。{page}ページ目"

        df.columns = [column[1] for column in layout]

        # Remove unnecessary columns
        for _, key, required in layout:
            if not required:
                del df[key]

        df_cleansed = df.applymap(removeNewLine)

        df_cleansed["category"] = category

        return df_cleansed

    def allergen_state_of(self, value: str) -> AllergenState:
        if value == "●":
            return AllergenState.CONTAIN
        elif value == "▲":
            return AllergenState.MAY_CONTAIN
        elif numpy.isnan(value):
            return AllergenState.NOT_CONTAIN
        else:
            raise ValueError(f"Unknown AllergenState {value}.")



# -*- coding: utf-8 -*-
from distutils.core import setup

package_dir = \
{'': 'src'}

packages = \
['sushi_allergy_parser']

package_data = \
{'': ['*']}

install_requires = \
['tabula-py>=1.4,<2.0']

setup_kwargs = {
    'name': 'sushi-allergy-parser',
    'version': '0.1.3',
    'description': 'sushi-allergy-parser is a parser for allergy infomation document of japanese conveyor-belt sushi chain. ',
    'long_description': "# sushi-allergy-parser\n\n`sushi-allergy-parser` is a parser for allergy infomation document of japanese conveyor-belt sushi chain. Currently, the following documents are supported.\n\n* スシロー (http://www3.akindo-sushiro.co.jp/pdf/menu/allergy.pdf)\n* くら寿司 (http://www.kura-corpo.co.jp/common/pdf/kura_allergen.pdf)\n\n\n## Requirements\n\n`sushi-allergy-parser` depends on [tabula-py](https://github.com/chezou/tabula-py), so you should install Java(7 or 8).\n\n## Installation\n\n```sh\n$ pip install sushi-allergy-parser\n```\n\n## Usage\n\n```python\n>>> from sushi_allergy_parser import SushiroAllergyParser\n>>> df = SushiroAllergyParser().parse()\n>>> type(df)\n<class 'pandas.core.frame.DataFrame'>\n>>> df\n    category       name          egg  ...      gelatin       sesame    cashewNut\n0        にぎり  合鴨ロースの煮込み  NOT_CONTAIN  ...  NOT_CONTAIN  NOT_CONTAIN  NOT_CONTAIN\n1        にぎり        赤えび  NOT_CONTAIN  ...  NOT_CONTAIN  NOT_CONTAIN  NOT_CONTAIN\n2        にぎり         あじ  NOT_CONTAIN  ...  NOT_CONTAIN  NOT_CONTAIN  NOT_CONTAIN\n3        にぎり  あじ(ネギ・生姜)  NOT_CONTAIN  ...  NOT_CONTAIN  NOT_CONTAIN  NOT_CONTAIN\n4        にぎり   穴子天ぷらにぎり  MAY_CONTAIN  ...  MAY_CONTAIN  MAY_CONTAIN  NOT_CONTAIN\n..       ...        ...          ...  ...          ...          ...          ...\n319      その他        天つゆ  NOT_CONTAIN  ...  MAY_CONTAIN  MAY_CONTAIN  NOT_CONTAIN\n320      その他       粉末緑茶  NOT_CONTAIN  ...  NOT_CONTAIN  NOT_CONTAIN  NOT_CONTAIN\n321      その他        ぽん酢  NOT_CONTAIN  ...  NOT_CONTAIN  MAY_CONTAIN  NOT_CONTAIN\n322      その他        わかめ  NOT_CONTAIN  ...  NOT_CONTAIN  NOT_CONTAIN  NOT_CONTAIN\n323      その他        わさび  MAY_CONTAIN  ...  MAY_CONTAIN  NOT_CONTAIN  NOT_CONTAIN\n```\n",
    'author': 't3yamoto',
    'author_email': '3yamoto.dev@gmail.com',
    'url': 'https://github.com/t3yamoto/sushi-allergy-parser',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
